@@ -88,13 +88,12 @@ _filter_mask_popover = None
 _filter_mask_menu = None
 _filter_add_popover = None
 _filter_add_menu = None
-_kb_shortcuts_popover = None
-_kb_shortcuts_menu = None
-_render_args_popove = None
+_render_args_popover = None
 _render_args_menu = None
 _kf_popover = None
 _kf_menu = None
-    
+
+
 # -------------------------------------------------- menuitems builder fuctions
 def add_menu_action(menu, label, item_id, data, callback, active=True, app=None):
     if active == True:
@@ -158,9 +157,9 @@ def new_popover(widget, menu, launcher):
 
     return popover
 
-def new_mouse_popover(widget, menu, rect):
+def new_mouse_popover(widget, menu, rect, position_type=Gtk.PositionType.BOTTOM):
     popover = Gtk.Popover.new_from_model(widget, menu)
-    popover.set_position(Gtk.PositionType(Gtk.PositionType.BOTTOM))
+    popover.set_position(Gtk.PositionType(position_type))
     popover.set_pointing_to(rect) 
     popover.show()
     
@@ -257,8 +256,6 @@ def all_tracks_menu_show(launcher, widget, callback):
     add_menu_action(maximize_section, _("Reset Track Heights"), "midbar.all.minimize", "resetheights" , callback)
     _all_tracks_menu.append_section(None, maximize_section)
 
-
-    
     activate_section = Gio.Menu.new()
     add_menu_action(activate_section, _("Activate All Tracks"), "midbar.all.allactive", "allactive" , callback)
     add_menu_action(activate_section, _("Activate Only Current Top Active Track"), "midbar.all.topactiveonly", "topactiveonly" , callback)
@@ -395,7 +392,6 @@ def monitor_view_popupmenu_show(launcher, widget, callback, callback_opacity):
         add_menu_action_all_items_radio(view_section, items_data, "monitor.viewimage", active_index, callback)
         _monitorview_menu.append_section(None, view_section)
 
-        # We are getting gtk warning here, look to fix
         _opacity_section = menu_clear_or_create(_opacity_section)
         _opacity_submenu = menu_clear_or_create(_opacity_submenu)
         items_data = [( _("100%"), "3"), ( _("80%"), "4"), ( _("50%"), "5"), ( _("20%"), "6")]
@@ -461,6 +457,14 @@ def media_hamburger_popover_show(launcher, widget, callback):
     add_menu_action(proxy_section, _("Render Proxy Files For All Media"), "mediapanel.proxyall", "render all proxies", callback)
     _media_panel_hamburger_menu.append_section(None, proxy_section)
 
+    order_section = Gio.Menu.new()
+    add_menu_action(order_section, _("Reverse Items Order"), "mediapanel.reverseorder", "reverse order", callback)
+    _media_panel_hamburger_menu.append_section(None, order_section)
+
+    action_section = Gio.Menu.new()
+    add_menu_action(action_section, _("Set Current Bin Graphics Default Length"), "mediapanel.setbindefault", "set bin default", callback)
+    _media_panel_hamburger_menu.append_section(None, action_section)
+
     select_section = Gio.Menu.new()
     add_menu_action(select_section, _("Select All"), "mediapanel.selectall", "select all", callback)
     add_menu_action(select_section, _("Select None"), "mediapanel.selectnone",  "select none", callback)
@@ -536,7 +540,7 @@ def media_file_popover_show(media_file, widget, x, y, callback):
         else:
             monitor_item_active = False
     else:
-            monitor_item_active = True
+        monitor_item_active = True
 
     if monitor_item_active == True:
         monitor_section = Gio.Menu.new()
@@ -901,25 +905,6 @@ def filter_add_popover_show(launcher, widget, clip, track, x, mltfiltersgroups,
     launcher.connect_launched_menu(_filter_add_popover)
     _filter_add_popover.show()
 
-def kb_shortcuts_popover_show(launcher, widget, data, callback):
-    global _kb_shortcuts_popover, _kb_shortcuts_menu
-
-    _kb_shortcuts_menu = menu_clear_or_create(_kb_shortcuts_menu)
-    
-    shortcuts_combo, dialog = data
-
-    if shortcuts_combo.get_active() < 2:
-        delete_active = False
-    else:
-        delete_active = True
-
-    main_section = Gio.Menu.new()
-    add_menu_action(main_section, _("Add Custom Shortcuts Group"), "kbdialog.addgroup", ("add", data), callback)
-    add_menu_action(main_section, _("Delete Active Custom Shortcuts Group"), "kbdialog.deletegroup", ("delete", data), callback)
-    _kb_shortcuts_menu.append_section(None, main_section)
-
-    _kb_shortcuts_popover = new_popover(widget, _kb_shortcuts_menu, launcher)
-
 def render_args_popover_show(launcher, widget, callback):
     global _render_args_popover, _render_args_menu
 
