@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,21 +48,59 @@ import shortcutsquickeffects
 import re
 import render
 import rotomask
+import targetactions
 import tlineaction
-# import tlinerender
 import tlinewidgets
+import tlineypage
+import trackaction
 import trimmodes
 import updater
 import projectaction
 import workflow
 
-
+# TODO: We should consider integrating some parts of this with targetactions.py
+# TODO:
+# TODO: As of this writing, targetactions.py has a superset of targetable
+# TODO: actions, as compared to keyevents.py, totally separate from any keyboard
+# TODO: event handling. There are a few new named target actions in there that
+# TODO: aren't available in here. There is also currently a lot code duplication
+# TODO: between the two modules. See targetactions.py for more details.
+# TODO:
+# TODO: At a minimum, if you add or modify any of the key actions in here,
+# TODO: please consider updating targetactions.py as well. Right now there
+# TODO: is a lot of duplication between these modules, and often a change
+# TODO: in one would warrant a change in the other.
+# TODO:
+# TODO: keyevents.py is all about handling key presses from the keyboard, and
+# TODO: routing those events to trigger actions in various parts of the program.
+# TODO:
+# TODO: targetactions.py is basically a bunch of zero-argument functions with
+# TODO: names based on the shortcut key names found here. It was created as part
+# TODO: of the USB HID work, so that USB jog/shuttle devices could have their
+# TODO: buttons target various actions within the program, without requiring
+# TODO: each USB driver to directly make connections to a dozen different parts
+# TODO: of the program to control it.
+# TODO:
+# TODO: So now we have two collections of shortcut key names which map to
+# TODO: basically the same actions, but in a different way. I originally wanted
+# TODO: to just use keyevents.py as the target for the USB driver actions, but
+# TODO: couldn't use it directly since this module is intertwined with the
+# TODO: main computer keyboard and its events.
+# TODO:
+# TODO: For now, I have integrated the new command targets from
+# TODO: targetactions.py into keyevents.py, both for completeness, and also as
+# TODO: a proof of concept as to how we might migrate some of the other code
+# TODO: in here over to call targetactions.py
+# TODO:
+# TODO:   -- Nathan Rosenquist (@ratherlargerobot)
+# TODO:      Feb 2022
 
 # ------------------------------------- keyboard events
 def key_down(widget, event):
     """
     Global key press listener.
     """
+
     # Handle ESCAPE.
     if event.keyval == Gdk.KEY_Escape:
         if editorstate.current_is_move_mode() == False:
@@ -110,7 +148,7 @@ def key_down(widget, event):
     if was_handled:
         return True
 
-    # Pressing timeline button obivously leaves user expecting
+    # Pressing timeline button obviously leaves user expecting
     # to have focus in timeline.
     if gui.monitor_switch.widget.has_focus() and timeline_visible():
         _handle_tline_key_event(event)
@@ -148,10 +186,10 @@ def key_down(widget, event):
     # Select all with CTRL + A in media panel.
     if event.keyval == Gdk.KEY_a:
         if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-            if gui.media_list_view.widget.has_focus() or gui.media_list_view.widget.get_focus_child() != None:
+            if gui.media_list_view.widget.has_focus() or gui.media_list_view.widget.get_focus_child() != None: 
                 gui.media_list_view.select_all()
                 return True
-        
+
     if event.keyval == Gdk.KEY_F11:
         menuactions.toggle_fullscreen()
         return True
@@ -199,9 +237,6 @@ def _handle_tline_key_event(event):
     if action == 'to_mark_in':
         monitorevent.to_mark_in_pressed()
         return True
-    #if action == 'tline_render_request':
-    #    tlinerender.render_all_segments()
-    #    return True
     if action == 'zoom_out':
         updater.zoom_out()
     if action == 'zoom_in':
@@ -212,9 +247,21 @@ def _handle_tline_key_event(event):
     if action == 'to_mark_out':
         monitorevent.to_mark_out_pressed()
         return True
+    if action == 'clear_mark_in':
+        targetactions.clear_mark_in()
+        return True
+    if action == 'clear_mark_out':
+        targetactions.clear_mark_out()
+        return True
     if action == 'clear_io_marks':
         monitorevent.marks_clear_pressed()
         return True
+    if action == 'play':
+        targetactions.play()
+        return True
+    if action == 'stop':
+        targetactions.stop()
+        return True
     if action == 'play_pause':
         if PLAYER().is_playing():
             monitorevent.stop_pressed()
@@ -227,6 +274,12 @@ def _handle_tline_key_event(event):
         else:
             monitorevent.start_marks_looping()
         return True
+    if action == 'display_clip_in_monitor':
+        targetactions.display_clip_in_monitor()
+        return True
+    if action == 'display_sequence_in_monitor':
+        targetactions.display_sequence_in_monitor()
+        return True
     if action == 'switch_monitor':
         updater.switch_monitor_display()
         return True
@@ -260,8 +313,12 @@ def _handle_tline_key_event(event):
     if action == 'trim_end':
         tlineaction.trim_end_pressed()
         return True
-        
-    # Key bindings for keyboard trimming.
+
+    # Key bindings for keyboard trimming
+    if action == 'toggle_track_output':
+        trackaction.toggle_track_output()
+        return True
+
     if editorstate.current_is_active_trim_mode() == True:
         if action == 'prev_frame':
             trimmodes.left_arrow_pressed((event.get_state() & Gdk.ModifierType.CONTROL_MASK))
@@ -329,11 +386,14 @@ def _handle_tline_key_event(event):
                 seek_amount = -1
             else:
                 seek_amount = 1
+
+            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
+                PLAYER().slowmo_seek_delta(seek_amount)
+                return True
                 
             if (event.get_state() & Gdk.ModifierType.SHIFT_MASK):
                 seek_amount = seek_amount * prefs.ffwd_rev_shift
-            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                seek_amount = seek_amount * prefs.ffwd_rev_ctrl
+
             if (event.get_state() & Gdk.ModifierType.LOCK_MASK):
                 seek_amount = seek_amount * prefs.ffwd_rev_caps
             PLAYER().seek_delta(seek_amount)
@@ -399,14 +459,14 @@ def _handle_tline_key_event(event):
         if action == 'to_start':
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
-            gui.editor_window.set_default_edit_tool()
+            gui.editor_window.tline_cursor_manager.set_default_edit_tool()
             PLAYER().seek_frame(0)
             _move_to_beginning()
             return True
         if action == 'to_end':
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
-            gui.editor_window.set_default_edit_tool()
+            gui.editor_window.tline_cursor_manager.set_default_edit_tool()
             PLAYER().seek_end()
             _move_to_end()
             return True
@@ -448,6 +508,12 @@ def _handle_extended_monitor_focus_events(event):
     if action == 'log_range':
         medialog.log_range_clicked()
         return True
+    if action == 'display_clip_in_monitor':
+        targetactions.display_clip_in_monitor()
+        return True
+    if action == 'display_sequence_in_monitor':
+        targetactions.display_sequence_in_monitor()
+        return True
     if action == 'switch_monitor':
         updater.switch_monitor_display()
         return True
@@ -507,13 +573,23 @@ def _get_shortcut_action(event):
 
 def _handle_configurable_global_events(event):
     action = _get_shortcut_action(event)
+    if action == 'tline_page_up':
+        tlineypage.page_up_key()
+        return True
+    if action == 'tline_page_down':
+        tlineypage.page_down_key()
+        return True
     if action == 'open_next':
         projectaction.open_next_media_item_in_monitor()
         return True
-    # if action == 'tline_render_request':
-    #    tlinerender.render_all_segments()
-    #    return True
-        
+    if action == 'open_prev':
+        projectaction.open_prev_media_item_in_monitor()
+        return True
+    if action == "append_from_bin":
+        if gui.media_list_view.widget.has_focus() or gui.media_list_view.widget.get_focus_child() != None: 
+            projectaction.append_selected_media_clips_into_timeline()
+            return True
+
     return False
     
 def _handle_clip_key_event(event):
@@ -531,10 +607,13 @@ def _handle_clip_key_event(event):
                 seek_amount = -1
             else:
                 seek_amount = 1
+            
+            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
+                PLAYER().slowmo_seek_delta(seek_amount)
+                return True
+                
             if (event.get_state() & Gdk.ModifierType.SHIFT_MASK):
                 seek_amount = seek_amount * prefs.ffwd_rev_shift
-            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                seek_amount = seek_amount * prefs.ffwd_rev_ctrl
             if (event.get_state() & Gdk.ModifierType.LOCK_MASK):
                 seek_amount = seek_amount * prefs.ffwd_rev_caps
             PLAYER().seek_delta(seek_amount)
@@ -564,6 +643,12 @@ def _handle_clip_key_event(event):
             else:
                  monitorevent.down_arrow_seek_on_monitor_clip()
                  return True
+        if action == 'play':
+            targetactions.play()
+            return True
+        if action == 'stop':
+            targetactions.stop()
+            return True
         if action == 'play_pause':
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
@@ -588,6 +673,12 @@ def _handle_clip_key_event(event):
         if action == 'to_mark_out':
             monitorevent.to_mark_out_pressed()
             return True
+        if action == 'clear_mark_in':
+            targetactions.clear_mark_in()
+            return True
+        if action == 'clear_mark_out':
+            targetactions.clear_mark_out()
+            return True
         if action == 'clear_io_marks':
             monitorevent.marks_clear_pressed()
             return True
@@ -616,11 +707,6 @@ def _handle_delete():
     if gui.editor_window.media_log_events_list_view.get_focus_child() != None:
         medialog.delete_selected()
         return True
-
-    # Delete tline render segment
-    #if gui.tline_render_strip.widget.has_focus() == True:
-    #    tlinerender.get_renderer().delete_selected_segment()
-    #    return True
     
     focus_editor = _get_focus_keyframe_editor(compositeeditor.keyframe_editor_widgets)
     if focus_editor != None:
@@ -651,6 +737,12 @@ def _handle_geometry_editor_keys(event):
                         return True
                     if event.keyval == Gdk.KEY_plus:
                         pass # not impl
+                    if action == 'play':
+                        targetactions.play()
+                        return True
+                    if action == 'stop':
+                        targetactions.stop()
+                        return True
                     if action == 'play_pause':
                         if PLAYER().is_playing():
                             monitorevent.stop_pressed()
@@ -669,6 +761,12 @@ def _handle_effects_editor_keys(event):
     action = _get_shortcut_action(event)
     focus_editor = _get_focus_keyframe_editor(clipeffectseditor.keyframe_editor_widgets)
     if focus_editor != None:
+        if action == 'play':
+            targetactions.play()
+            return True
+        if action == 'stop':
+            targetactions.stop()
+            return True
         if action == 'play_pause':
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
@@ -687,10 +785,13 @@ def _handle_effects_editor_keys(event):
                 seek_amount = -1
             else:
                 seek_amount = 1
+            
+            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
+                PLAYER().slowmo_seek_delta(seek_amount)
+                return True
+                
             if (event.get_state() & Gdk.ModifierType.SHIFT_MASK):
                 seek_amount = seek_amount * prefs.ffwd_rev_shift
-            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                seek_amount = seek_amount * prefs.ffwd_rev_ctrl
             if (event.get_state() & Gdk.ModifierType.LOCK_MASK):
                 seek_amount = seek_amount * prefs.ffwd_rev_caps
             PLAYER().seek_delta(seek_amount)
