@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@ from editorstate import current_is_move_mode
 from editorstate import MONITOR_MEDIA_FILE
 import gui
 import guicomponents
+import guipopover
 import movemodes
 import trimmodes
 import updater
@@ -207,6 +208,26 @@ def mark_out_pressed():
     _do_marks_update()
     updater.display_marks_tc()
 
+def mark_in_clear_pressed():
+    if timeline_visible():
+        trimmodes.set_no_edit_trim_mode()
+        PLAYER().producer.mark_in = -1
+    else:
+        current_sequence().monitor_clip.mark_in = -1
+
+    _do_marks_update()
+    updater.display_marks_tc()
+
+def mark_out_clear_pressed():
+    if timeline_visible():
+        trimmodes.set_no_edit_trim_mode()
+        PLAYER().producer.mark_out = -1
+    else:
+        current_sequence().monitor_clip.mark_out = -1
+
+    _do_marks_update()
+    updater.display_marks_tc()
+
 def marks_clear_pressed():
     if timeline_visible():
         trimmodes.set_no_edit_trim_mode()
@@ -321,25 +342,12 @@ def select_prev_clip_for_filter_edit():
     clipeffectseditor.set_clip(clip, track, range_in)
 
 # --------------------------------------------------------- trim view
-def trim_view_menu_launched(launcher, event):
-    guicomponents.get_trim_view_popupmenu(launcher, event, _trim_view_menu_item_activated)
-
-def _trim_view_menu_item_activated(widget, msg):
-    if msg == "matchclear":
-        gui.monitor_widget.set_default_view_force()
-        return
-    if msg == "clipframematch":
-        import tlineaction  # if this is on top level gmic tool get circular import
-        clip = tlineaction._get_new_clip_from_clip_monitor()
-        if clip == None:
-            return
-        frame = PLAYER().current_frame()
-        gui.monitor_widget.set_frame_match_view(clip, frame)
-        return
-
-    if widget.get_active() == False:
-        return
+def trim_view_menu_launched(launcher, widget, event):
+    guipopover.trim_view_popover_show(launcher, widget, _trim_view_menu_item_activated)
 
+def _trim_view_menu_item_activated(action, new_value_variant):
+    msg = new_value_variant.get_string()
+        
     if msg == "trimon":
         editorstate.show_trim_view = appconsts.TRIM_VIEW_ON
         editorpersistance.prefs.trim_view_default = appconsts.TRIM_VIEW_ON
@@ -357,6 +365,9 @@ def _trim_view_menu_item_activated(widget, msg):
         editorpersistance.prefs.trim_view_default = appconsts.TRIM_VIEW_OFF
         editorpersistance.save()
 
+    action.set_state(new_value_variant)
+    guipopover._trimview_popover.hide()
+        
 def _show_trimview_info():
     editorpersistance.prefs.trim_view_message_shown = True
     editorpersistance.save()
