@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -63,7 +63,7 @@ MONITOR_INDICATOR_COLOR_MATCH = utils.get_cairo_color_tuple_255_rgb(21, 71, 105)
 
 FRAME_MATCH_VIEW_COLOR = (0.3, 0.3, 0.3)
 
-# Continuos match frame update
+# Continuous match frame update
 CONTINUOS_UPDATE_PAUSE = 0.2
 _last_render_time = 0.0
 _producer = None
@@ -72,9 +72,11 @@ _frame_write_on = False
             
 _widget = None
 
+
 def _get_match_frame_path():
     return userfolders.get_cache_dir() + appconsts.TRIM_VIEW_DIR + "/" + MATCH_FRAME
-        
+
+
 class MonitorWidget:
     
     def __init__(self):
@@ -91,16 +93,16 @@ class MonitorWidget:
         self.slip_clip_length = -1
 
         self.clip_name = "clip name"
-    
+
         self.match_not_updateble = False # Roll and Slip need this flag to know if surface updates needed
-        
+
         # top row
         self.top_row = Gtk.HBox()
         
         self.top_edge_panel = cairoarea.CairoDrawableArea2(1, 1, self._draw_top_panel, use_widget_bg=False)
         self.top_edge_panel.press_func = self._press_event
         self.top_row.pack_start(self.top_edge_panel, True, True,0)
-        
+
         # mid row
         self.mid_row = Gtk.HBox()
 
@@ -165,11 +167,6 @@ class MonitorWidget:
 
         if self.view == FRAME_MATCH_VIEW and force_default_mode==False:
             return
-            
-        # Refreshing player while rendering overwrites file on disk and loses 
-        # previous rendered data. 
-        if PLAYER().is_rendering:
-            return
 
         # Delete match frame
         try:
@@ -192,10 +189,6 @@ class MonitorWidget:
         PLAYER().refresh()
 
     def set_frame_match_view(self, match_clip, frame):        
-        # Refreshing while rendering overwrites file on disk and loses 
-        # previous rendered data. 
-        if PLAYER().is_rendering:
-            return
 
         # Delete match frame
         try:
@@ -223,11 +216,6 @@ class MonitorWidget:
     def set_start_trim_view(self, match_clip, edit_clip_start):
         if self.is_active(True) == False:
             return
-
-        # Refreshing while rendering overwrites file on disk and loses 
-        # previous rendered data. 
-        if PLAYER().is_rendering:
-            return
            
         self.view = START_TRIM_VIEW
         self.match_frame_surface = None
@@ -260,11 +248,6 @@ class MonitorWidget:
     def set_end_trim_view(self, match_clip, edit_clip_start):
         if self.is_active(True) == False:
             return
-
-        # Refreshing while rendering overwrites file on disk and loses 
-        # previous rendered data. 
-        if PLAYER().is_rendering:
-            return
         
         self.view = END_TRIM_VIEW
         self.match_frame_surface = None
@@ -288,7 +271,7 @@ class MonitorWidget:
             self.match_not_updateble = True
             self.create_pattern_producer_match_frame()
             return
-            
+
         self.match_frame = match_clip.clip_in
         self.match_not_updateble = False
         data = (match_clip.path, match_clip.clip_in, MATCH_FRAME, self.match_frame_write_complete)
@@ -297,11 +280,6 @@ class MonitorWidget:
     def set_roll_trim_right_active_view(self, match_clip, edit_clip_start):
         if self.is_active() == False:
             return
-
-        # Refreshing while rendering overwrites file on disk and loses 
-        # previous rendered data. 
-        if PLAYER().is_rendering:
-            return
         
         self.view = ROLL_TRIM_RIGHT_ACTIVE_VIEW
         self.match_frame_surface = None
@@ -334,11 +312,6 @@ class MonitorWidget:
     def set_roll_trim_left_active_view(self, match_clip, edit_clip_start):
         if self.is_active() == False:
             return
-            
-        # Refreshing while rendering overwrites file on disk and loses 
-        # previous rendered data. 
-        if PLAYER().is_rendering:
-            return
         
         self.view = ROLL_TRIM_LEFT_ACTIVE_VIEW
         self.match_frame_surface = None
@@ -372,11 +345,6 @@ class MonitorWidget:
         if self.is_active() == False:
             return
 
-        # Refreshing while rendering overwrites file on disk and loses 
-        # previous rendered data. 
-        if PLAYER().is_rendering:
-            return
-
         self.view = SLIP_TRIM_RIGHT_ACTIVE_VIEW
         self.match_frame_surface = None
         self.edit_clip_start_on_tline = 0 # We're using tiline frames just as units to get edit deltas and displayed clip tc
@@ -410,11 +378,6 @@ class MonitorWidget:
         if self.is_active() == False:
             return
 
-        # Refreshing while rendering overwrites file on disk and loses 
-        # previous rendered data. 
-        if PLAYER().is_rendering:
-            return
-    
         self.view = SLIP_TRIM_LEFT_ACTIVE_VIEW
         self.match_frame_surface = None
         self.edit_clip_start_on_tline = 0 # We're using tiline frames just as units to get edit deltas and displayed clip tc
@@ -525,11 +488,11 @@ class MonitorWidget:
     def one_roll_mouse_release(self, edit_tline_frame, edit_delta):
         if self.is_active() == False:
             return
-            
         self.edit_tline_frame = edit_tline_frame
         if self.view == START_TRIM_VIEW: # were computing displayed edit side TC 
                                          # from current_tline_frame - clip_start_frame and clip_start_frame changes now if START_TRIM_VIEW
             self.edit_clip_start_on_tline = self.edit_clip_start_on_tline - edit_delta
+
         self.edit_delta = None
         self.bottom_edge_panel.queue_draw()
     
@@ -547,7 +510,7 @@ class MonitorWidget:
         GLib.timeout_add(0, self._draw_displays)
             
     def _draw_displays(self):
-        # This is called from GUI prosess, we can do Gtk calls.
+        # This is called from GUI process, we can do Gtk calls.
         self.left_display.queue_draw()
         self.right_display.queue_draw()
 
@@ -853,7 +816,7 @@ class MonitorWidget:
 
 
     
-# ---------------------------------------------------------------------------------- match frame cration
+# ---------------------------------------------------------------------------------- match frame creation
 def _launch_match_frame_writer(data):
     match_clip_path, clip_frame, frame_name, callback = data        
 
@@ -915,7 +878,6 @@ class MatchSurfaceCreator(threading.Thread):
         threading.Thread.__init__(self)
         
     def run(self):
-        print("MatchSurfaceCreator.run")
         # Create new producer to get mlt frame data
         while _producer == None:
             print("MatchSurfaceCreator: waiting for _producer")
