@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -149,7 +149,7 @@ def save_project(project, file_path, changed_profile_desc=None):
     project_proxy_mode = s_proj.proxy_data.proxy_mode
     proxy_path_dict = {}
 
-    # Replace media file objects with pickleable copys
+    # Replace media file objects with pickleable copies
     media_files = {}
     for k, v in s_proj.media_files.items():
         s_media_file = copy.copy(v)
@@ -377,7 +377,7 @@ def _save_changed_xml_file(s_media_file, new_profile):
 
     folder = userfolders.get_render_dir()
     uuid_str = hashlib.md5(str(os.urandom(32)).encode('utf-8')).hexdigest()
-    new_xml_file_path = folder + "/"+ uuid_str + ".xml"
+    new_xml_file_path = folder + uuid_str + ".xml"
 
     with atomicfile.AtomicFileWriter(new_xml_file_path, "w") as afw:
         new_xml_file = afw.get_file()
@@ -482,7 +482,12 @@ def load_project(file_path, icons_and_thumnails=True, relinker_load=False):
 
     all_clips = {}
     sync_clips = []
-
+    
+    # Bins need fix for added attr.
+    for bin in project.bins:
+        persistancecompat.FIX_MISSING_BIN_ATTRS(bin)
+    persistancecompat.FIX_MISSING_BIN_ATTRS(project.c_bin)
+ 
     if icons_and_thumnails == True:
         _show_msg(_("Loading icons"))
         for k, media_file in project.media_files.items():
@@ -530,7 +535,7 @@ def fill_sequence_mlt(seq, SAVEFILE_VERSION):
     dead_compositors = 0
                 
     for py_compositor in seq.compositors:
-            # Keeping backwards compability
+            # Keeping backwards compatibility
             persistancecompat.FIX_MISSING_COMPOSITOR_ATTRS(py_compositor)
 
             # Create new compositor object
@@ -645,7 +650,11 @@ def fill_track_mlt(mlt_track, py_track):
                             clip.container_data.clear_rendered_media()
 
             mlt_clip = sequence.create_file_producer_clip(clip.path, None, False, clip.ttl)
-            
+
+            # Image clips longer then 15000 frames need to have their length property set.
+            if clip.media_type == appconsts.IMAGE and clip.clip_out > 15000:
+                mlt_clip.set("length", int(clip.clip_out - 1)) # -1, out is inclusive
+
             if mlt_clip == None:
                 raise FileProducerNotFoundError(orig_path)
 
@@ -677,7 +686,7 @@ def fill_track_mlt(mlt_track, py_track):
         if append_created == True:
             append_clip(mlt_track, mlt_clip, clip.clip_in, clip.clip_out)
 
-        # Save refences to recreate sync relations after all clips loaded
+        # Save references to recreate sync relations after all clips loaded
         global all_clips, sync_clips
         all_clips[mlt_clip.id] = mlt_clip
         if mlt_clip.sync_data != None:
@@ -806,7 +815,7 @@ def get_img_seq_relative_path(project_file_path, asset_path):
     return NOT_FOUND # no relative path found
         
     
-# ------------------------------------------------------- backwards compability
+# ------------------------------------------------------- backwards compatibility
 def _fix_wipe_relative_path(compositor):
     if compositor.type_id == "##wipe": # Wipe may have user luma and needs to be looked up relatively
         _set_wipe_res_path(compositor, "resource")
