@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,7 +48,6 @@ import app
 import audiowaveformrenderer
 import appconsts
 import batchrendering
-import containerprogramedit
 import clipeffectseditor
 import compositeeditor
 import containerclip
@@ -56,6 +55,7 @@ import dialogs
 import dialogutils
 import gui
 import guicomponents
+import guipopover
 import guiutils
 import edit
 import editorstate
@@ -73,27 +73,24 @@ import mediaplugin
 import modesetting
 import movemodes
 import mltprofiles
+import patternproducer
 import persistance
 import projectdata
 import projectinfogui
 import projectmediaimport
 import propertyparse
+import projectdatavault
 import proxyediting
 import render
 import renderconsumer
 import rendergui
 import sequence
-# import tlinerender
+import tlinewidgets
 import undo
 import updater
 import userfolders
 import utils
 
-media_panel_popup_menu = Gtk.Menu()
-bin_popup_menu = Gtk.Menu()
-sequence_popup_menu = Gtk.Menu()
-hamburger_popup_menu = Gtk.Menu()
-compositing_mode_menu = Gtk.Menu()
 
 save_time = None
 save_icon_remove_event_id = None
@@ -117,8 +114,12 @@ class LoadThread(threading.Thread):
         self.block_recent_files = block_recent_files
         self.is_first_video_load = is_first_video_load
         self.is_autosave_load = is_autosave_load
+        self.replace_media_file_path = None
         threading.Thread.__init__(self)
 
+    def set_replace_media_path(self, replace_media_file_path):
+        self.replace_media_file_path = replace_media_file_path
+
     def run(self):
         ticker = utils.Ticker(_load_pulse_bar, 0.15)
         ticker.start_ticker()
@@ -137,7 +138,7 @@ class LoadThread(threading.Thread):
             GLib.idle_add(self._exit_on_file_not_found_error, e, ticker)
             return
         except persistance.ProjectProfileNotFoundError as e:
-            GLib.idle_add(self._exit_on_profile_file_not_found_error, e, ticker)
+            GLib.idle_add(self._exit_on_profile_file_not_found_error, e, ticker, old_project)
             return
 
         guiutils.update_text_idle(self.dialog.info, _("Opening"))
@@ -184,7 +185,7 @@ class LoadThread(threading.Thread):
             render.set_saved_gui_selections(selections)
         updater.set_info_icon(None)
 
-        if self.is_autosave_load == False: # project loaded with autosave needs to keep its last_save_path data.
+        if self.is_autosave_load == False and self.is_first_video_load == False: # project loaded with autosave needs to keep its last_save_path data.
             # If project file is moved since last save we need to update last_save_path property and save to get everything working as expected.
             if self.filename != editorstate.project.last_save_path:
                 print("Project file moved since last save, save with updated last_save_path data.")
@@ -192,7 +193,7 @@ class LoadThread(threading.Thread):
                 editorstate.project.name = os.path.basename(self.filename)
                 _save_project_in_last_saved_path()
         else:
-            print("autosave load")
+            pass
 
         self.dialog.destroy()
         gui.tline_canvas.connect_mouse_events() # mouse events during load cause crashes because there is no data to handle
@@ -202,6 +203,19 @@ class LoadThread(threading.Thread):
         # Show info on killed compositors.
         if persistance.dead_compositors > 0:
             dialogs.show_dead_compositors(persistance.dead_compositors)
+        
+        # Update icons for 'replace media' feature.
+        if self.replace_media_file_path != None: 
+            (icon_path, length, info) = projectdata.thumbnailer.write_image(self.replace_media_file_path)
+            media_file = PROJECT().get_media_file_for_path(self.replace_media_file_path)
+            media_file.info = info
+            media_file.icon_path = icon_path
+            media_file.create_icon()
+            
+            tlinewidgets.update_clip_thumbnail(media_file)
+
+            gui.media_list_view.widget.queue_draw()
+            updater.repaint_tline()
 
     def _exit_on_file_not_found_error(self, e, ticker):
         print("LoadThread.run() - FileProducerNotFoundError")
@@ -229,7 +243,7 @@ class LoadThread(threading.Thread):
         # We exit to same state as when app is first opened.
         app.open_project(projectdata.get_default_project())
 
-    def _exit_on_profile_file_not_found_error(self, e, ticker):
+    def _exit_on_profile_file_not_found_error(self, e, ticker, old_project):
         self._error_stop(self.dialog, ticker)
         primary_txt = _("Profile with Description: '") + e.value  + _("' was not found on load!")
         secondary_txt = _("It is possible to load the project by creating a User Profile with exactly the same Description\nas the missing profile. ") + "\n\n" + \
@@ -295,7 +309,7 @@ class AddMediaFilesThread(threading.Thread):
                     PROJECT().add_media_file(new_file, self.compound_clip_name, target_bin)
                     succes_new_file = new_file
                 except projectdata.ProducerNotValidError as err:
-                    GLib.idle_add(self._not_valid_producer, err.value)
+                    GLib.idle_add(self._not_valid_producer, err)
 
             self.list_view_update_done = False
             GLib.idle_add(self._list_view_update)
@@ -335,7 +349,7 @@ class AddMediaFilesThread(threading.Thread):
         update_current_bin_files_count()
         _enable_save()
 
-        normal_cursor = Gdk.Cursor.new(Gdk.CursorType.LEFT_PTR) #RTL
+        normal_cursor = Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.LEFT_PTR) #RTL
         gui.editor_window.window.get_window().set_cursor(normal_cursor)
         gui.editor_window.bin_info.display_bin_info()
         
@@ -360,13 +374,45 @@ class AddMediaFilesThread(threading.Thread):
         self.post_load_update_done = True
         
     def _change_cursor_watch(self):
-        watch = Gdk.Cursor.new(Gdk.CursorType.WATCH)
+        watch = Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.WATCH)
         gui.editor_window.window.get_window().set_cursor(watch)
 
-    def _not_valid_producer(self, err_value):
-        dialogs.not_valid_producer_dialog(err_value, gui.editor_window.window)
+    def _not_valid_producer(self, err):
+        dialogs.not_valid_producer_dialog(err, gui.editor_window.window)
+
+
+
+class AddTitleItemThread(threading.Thread):
+    
+    def __init__(self, name, filepath, title_data, completed_callback):
+        threading.Thread.__init__(self)
+        self.name = name
+        self.filepath = filepath
+        self.title_data = title_data
+        self.completed_callback = completed_callback
+
+    def run(self): 
+        target_bin = PROJECT().c_bin
+        PROJECT().add_title_item(self.name, self.filepath, self.title_data, target_bin)
+
+        self.list_view_update_done = False
+        GLib.idle_add(self._list_view_update)
+        while self.list_view_update_done == False:
+            time.sleep(0.05)
+
+        project_event = projectdata.ProjectEvent(projectdata.EVENT_MEDIA_ADDED, str(1))
+        PROJECT().events.append(project_event)
+
+        self.completed_callback()
+
+    def _list_view_update(self):
+        gui.media_list_view.fill_data_model()
+        max_val = gui.editor_window.media_scroll_window.get_vadjustment().get_upper()
+        gui.editor_window.media_scroll_window.get_vadjustment().set_value(max_val)
+        self.list_view_update_done = True
+
+
 
-                    
 class UpdateMediaLengthsThread(threading.Thread):
     
     def __init__(self, dialog):
@@ -433,12 +479,12 @@ def _not_matching_media_info_callback(dialog, response_id, media_file):
         path = userfolders.get_cache_dir() + "/" + PROJECT().name
         PROJECT().update_media_lengths_on_load = True
         
-        # Save this in case project was saved before firts video loaded
+        # Save this in case project was saved before first video loaded
         global first_video_load_project_save_path
         first_video_load_project_save_path = PROJECT().last_save_path
         
         persistance.save_project(PROJECT(), path, profile.description()) #<----- HERE
-        
+
         actually_load_project(path, False, True)
 
 def _load_pulse_bar():
@@ -448,22 +494,29 @@ def _enable_save():
     if PROJECT().last_save_path != None:
         gui.editor_window.uimanager.get_widget("/MenuBar/FileMenu/Save").set_sensitive(True)
 
+    return False # Can be called with GLib.idle_add, docs said "If the function returns GLib.SOURCE_REMOVE or False it is automatically removed from the list of event sources and will not be called again."
 
 # ---------------------------------- project: new, load, save
 def new_project():
     dialogs.new_project_dialog(_new_project_dialog_callback)
 
-def _new_project_dialog_callback(dialog, response_id, profile_combo, tracks_select):
+def _new_project_dialog_callback(dialog, response_id, profile_combo, tracks_select, vault_combo):
 
     v_tracks, a_tracks = tracks_select.get_tracks()
     
     if response_id == Gtk.ResponseType.ACCEPT:
         profile_name = profile_combo.get_selected()
         profile_index = mltprofiles.get_index_for_name(profile_name)
-        app.new_project(profile_index, v_tracks, a_tracks)
+        vault_folder = projectdatavault.get_vault_folder_for_index(vault_combo.get_active())
+
         dialog.destroy()
+                
+        app.new_project(profile_index, v_tracks, a_tracks)
+
         project_event = projectdata.ProjectEvent(projectdata.EVENT_CREATED_BY_NEW_DIALOG, None)
         PROJECT().events.append(project_event)
+        PROJECT().create_vault_folder_data(vault_folder)
+        projectdatavault.create_project_data_folders()
     else:
         dialog.destroy()
 
@@ -491,7 +544,10 @@ def _close_dialog_callback(dialog, response_id, no_dialog_project_close=False):
             pass
         elif response_id ==  Gtk.ResponseType.YES:# "Save"
             if editorstate.PROJECT().last_save_path != None:
+                
                 persistance.save_project(editorstate.PROJECT(), editorstate.PROJECT().last_save_path)
+                
+                projectdatavault.project_saved(PROJECT().last_save_path)
             else:
                 dialogutils.warning_message(_("Project has not been saved previously"), 
                                         _("Save project with File -> Save As before closing."),
@@ -505,16 +561,21 @@ def _close_dialog_callback(dialog, response_id, no_dialog_project_close=False):
     sequence.VIDEO_TRACKS_COUNT = appconsts.INIT_V_TRACKS
 
     new_project = projectdata.get_default_project()
+    vault_folder = projectdatavault.get_active_vault_folder()
+    new_project.create_vault_folder_data(vault_folder)
     app.open_project(new_project)
-    
-def actually_load_project(filename, block_recent_files=False, is_first_video_load=False, is_autosave_load=False):
+    projectdatavault.create_project_data_folders()
+
+def actually_load_project(filename, block_recent_files=False, is_first_video_load=False, is_autosave_load=False, replace_media_file_path=None):
     gui.tline_canvas.disconnect_mouse_events() # mouse events dutring load cause crashes because there is no data to handle
-    updater.set_info_icon(Gtk.STOCK_OPEN)
+    updater.set_info_icon("document-open")
 
     dialog = dialogs.load_dialog()
     persistance.load_dialog = dialog
 
     load_launch = LoadThread(dialog, filename, block_recent_files, is_first_video_load, is_autosave_load)
+    if replace_media_file != None:
+        load_launch.set_replace_media_path(replace_media_file_path)
     load_launch.start()
 
 def save_project():
@@ -530,6 +591,7 @@ def _save_project_in_last_saved_path():
     try:
         
         persistance.save_project(PROJECT(), PROJECT().last_save_path) #<----- HERE
+        projectdatavault.project_saved( PROJECT().last_save_path)
         
     except IOError as ioe:
         updater.set_info_icon(None)
@@ -574,7 +636,8 @@ def _save_as_dialog_callback(dialog, response_id):
         try:
             
             persistance.save_project(PROJECT(), PROJECT().last_save_path) #<----- HERE
-            
+            projectdatavault.project_saved( PROJECT().last_save_path)
+        
         except IOError as ioe:
             dialog.destroy()
             updater.set_info_icon(None)
@@ -635,7 +698,8 @@ def _change_project_profile_callback(dialog, response_id, profile_combo, out_fol
         PROJECT().name  = name
         
         persistance.save_project(PROJECT(), path, profile.description()) #<----- HERE
-
+        print("_change_project_profile_callback")
+            
         project_event = projectdata.ProjectEvent(projectdata.EVENT_PROFILE_CHANGED_SAVE, str(profile.description()))
         PROJECT().events.append(project_event)
         
@@ -684,6 +748,8 @@ def _open_recent_shutdown_dialog_callback(dialog, response_id, path):
     elif response_id ==  Gtk.ResponseType.YES:# "Save"
         if editorstate.PROJECT().last_save_path != None:
             persistance.save_project(editorstate.PROJECT(), editorstate.PROJECT().last_save_path)
+            projectdatavault.project_saved( PROJECT().last_save_path)
+            print("_open_recent_shutdown_dialog_callback")
         else:
             dialogutils.warning_message(_("Project has not been saved previously"), 
                                     _("Save project with File -> Save As before closing."),
@@ -870,44 +936,10 @@ def _write_out_render_item(single_render_item_item):
     return True
 
 # ----------------------------------- media files
-def hamburger_pressed(widget, event):
-    hamburger_menu = hamburger_popup_menu
+def hamburger_pressed(launcher, widget, event, data):
+    guipopover.media_hamburger_popover_show(launcher, widget, _hamburger_menu_item_selected)
     
-    guiutils.remove_children(hamburger_menu)
-
-    hamburger_menu.add(guiutils.get_menu_item(_("Render Proxy Files For Selected Media"), _hamburger_menu_item_selected, "render proxies", ))
-    hamburger_menu.add(guiutils.get_menu_item(_("Render Proxy Files For All Media"), _hamburger_menu_item_selected, "render all proxies", ))
-    guiutils.add_separetor(hamburger_menu)
-    hamburger_menu.add(guiutils.get_menu_item(_("Select All"), _hamburger_menu_item_selected, "select all"))
-    hamburger_menu.add(guiutils.get_menu_item(_("Select None"), _hamburger_menu_item_selected, "select none"))
-
-    move_menu_item = Gtk.MenuItem(_("Move Selected Media To Bin"))
-    move_menu = Gtk.Menu()
-    if len(PROJECT().bins) == 1:
-        item = guiutils.get_menu_item(_("No Target Bins"), _hamburger_menu_item_selected, "dummy")
-        item.set_sensitive(False)
-        move_menu.add(item)
-    else:
-        index = 0
-        for media_bin in PROJECT().bins:
-            if media_bin == PROJECT().c_bin:
-                index = index + 1
-                continue
-            item = guiutils.get_menu_item(media_bin.name, _hamburger_menu_item_selected, str(index))
-            move_menu.add(item)
-            item.show()
-            index = index + 1
-    move_menu_item.set_submenu(move_menu)
-    hamburger_menu.add(move_menu_item)
-    move_menu_item.show()
-    
-    guiutils.add_separetor(hamburger_menu)
-    hamburger_menu.add(guiutils.get_menu_item(_("Append All Media to Timeline"), _hamburger_menu_item_selected, "append all"))
-    hamburger_menu.add(guiutils.get_menu_item(_("Append Selected Media to Timeline"), _hamburger_menu_item_selected, "append selected"))
-    
-    hamburger_menu.popup(None, None, None, None, event.button, event.time)
-
-def _hamburger_menu_item_selected(widget, msg):
+def _hamburger_menu_item_selected(action, variant, msg):
     if msg == "render proxies":
         proxyediting.create_proxy_files_pressed()
     elif msg == "render all proxies":
@@ -920,6 +952,13 @@ def _hamburger_menu_item_selected(widget, msg):
         append_all_media_clips_into_timeline()
     elif msg == "append selected":
         append_selected_media_clips_into_timeline()
+    elif msg == "reverse order":
+        reverse_current_bin_media_order()
+    elif msg == "set bin default":
+        current_default_length = PROJECT().get_current_bin_graphics_default_length()
+        dialogs.set_bin_grfx_default_length_dialog( current_bin(),
+                                                    current_default_length,
+                                                    _set_bin_grfx_default_length_callback)
     else:
         target_bin_index = int(msg)
         
@@ -930,21 +969,18 @@ def _hamburger_menu_item_selected(widget, msg):
         
         move_files_to_bin(target_bin_index, moved_files)
 
-def media_panel_popup_requested(event):
-    panel_menu = media_panel_popup_menu
-    
-    guiutils.remove_children(panel_menu)
+def media_panel_popup_requested(widget, event):
+    guipopover. media_panel_popover_show(widget, event.x, event.y, _media_panel_menu_item_selected)
 
-    panel_menu.add(guiutils.get_menu_item(_("Add Video, Audio or Image..."), _media_panel_menu_item_selected, "add media", ))
-    panel_menu.add(guiutils.get_menu_item(_("Add Image Sequence..."), _media_panel_menu_item_selected, "add image sequence"))
-    
-    panel_menu.popup(None, None, None, None, event.button, event.time)
-
-def _media_panel_menu_item_selected(widget, msg):
+def _media_panel_menu_item_selected(action, variant, msg):
     if msg == "add media":
         add_media_files()
     elif msg == "add image sequence":
         add_image_sequence()
+    elif msg == "add generator":
+        mediaplugin.show_add_media_plugin_window()
+    elif msg == "add color clip":
+        patternproducer.create_color_clip()
 
 def media_panel_double_click(media_file):
     global _media_panel_double_click_counter
@@ -968,8 +1004,8 @@ def _open_files_dialog_cb(file_select, response_id):
     if len(filenames) == 0:
         return
     
-    # We're disallowing opening .mlt or .xml files as media beause MLTs behaviour of overwriting project profile properties
-    # when opening MLT XML files as nedia
+    # We're disallowing opening .mlt or .xml files as media because MLTs behaviour of overwriting project profile properties
+    # when opening MLT XML files as media
     # Underlying reason: https://github.com/mltframework/mlt/issues/212
     mlt_files_deleted = False
     last_non_matching_profile = None
@@ -1021,7 +1057,7 @@ def _add_image_sequence_callback(dialog, response_id, data):
     resource_name_str = utils.get_img_seq_resource_name(frame_file)
         
     # Detect highest file so that we know the length of the producer.
-    # FIX: this fails if two similarily numbered sequences in same dir and both have same substring in frame name
+    # FIX: this fails if two similarly numbered sequences in same dir and both have same substring in frame name
     number_index = file_name.find(number_part)
     path_name_part = file_name[0:number_index]
     onlyfiles = [ f for f in listdir(folder) if isfile(join(folder,f)) ]
@@ -1165,6 +1201,23 @@ def delete_media_files(force_delete=False):
     _enable_save()
     gui.editor_window.bin_info.display_bin_info()
 
+def replace_media_file(media_file):
+    dialogs.replace_media_dialog(media_file, _replace_media_callback)
+
+def _replace_media_callback(dialog, media_file, replace_file):
+    dialog.destroy()
+    
+    if media_file.type != sequence.get_media_type(replace_file):
+        primary_txt = _("Cannot complete media replace action")
+        secondary_txt = _("Original and replace media are not of the same media type.")
+        parent_window = gui.editor_window.window
+        dialogutils.info_message(primary_txt, secondary_txt, parent_window)
+        return
+
+    temp_saved_project_path = medialinker.replace_single_file(PROJECT(), media_file.path, replace_file)
+    actually_load_project(temp_saved_project_path, block_recent_files=False, is_first_video_load=False, is_autosave_load=False, replace_media_file_path=replace_file)
+
+    
 def _proxy_delete_warning_callback(dialog, response_id):
     dialog.destroy()
     if response_id == Gtk.ResponseType.OK:
@@ -1193,7 +1246,31 @@ def open_next_media_item_in_monitor():
     gui.media_list_view.update_selected_bg_colors()
     updater.set_and_display_monitor_media_file(media_file)
     gui.pos_bar.widget.grab_focus()
-            
+
+def open_prev_media_item_in_monitor():
+    # Get id for prev media file
+    selection = gui.media_list_view.get_selected_media_objects()
+    if len(selection) < 1:
+        try: # Nothing selected, get first media item
+            next_media_file_id = current_bin().file_ids[0]
+        except:
+            return # bin is empty
+    else: # Get prev media item from selection
+        current_media_file_id = selection[0].media_file.id
+        prev_media_index = current_bin().file_ids.index(current_media_file_id) - 1
+        if prev_media_index == -1:
+            prev_media_index = 0
+
+        prev_media_file_id = current_bin().file_ids[prev_media_index]
+
+    # Get media file, select it and show in monitor
+    media_file = PROJECT().media_files[prev_media_file_id]
+
+    gui.media_list_view.select_media_file(media_file)
+    gui.media_list_view.update_selected_bg_colors()
+    updater.set_and_display_monitor_media_file(media_file)
+    gui.pos_bar.widget.grab_focus()
+
 def display_media_file_rename_dialog(media_file):
     dialogs.new_media_name_dialog(media_file_name_edited, media_file)
 
@@ -1294,22 +1371,30 @@ def unused_media():
         unused.append(media_item)
     return unused
 
-def media_filtering_select_pressed(widget, event):
-    guicomponents.get_file_filter_popup_menu(widget, event, _media_filtering_selector_item_activated)
+def media_filtering_select_pressed(launcher, widget, event):
+    guipopover.file_filter_popover_show(launcher, widget,  _media_filtering_selector_item_activated)
 
-def _media_filtering_selector_item_activated(selector, index):
+def _media_filtering_selector_item_activated(action, new_value_variant):
+    index = int( new_value_variant.get_string())
     gui.media_view_filter_selector.set_pixbuf(index)
     
-    # Const value correspond with indexes here
+    # Const values at appconst,py correspond with indexes here.
     editorstate.media_view_filter = index
     gui.media_list_view.fill_data_model()
 
-def columns_count_launch_pressed(widget, event):
-    guicomponents.get_columns_count_popup_menu(event, _columns_count_item_selected)
+    action.set_state(new_value_variant)
+    guipopover._file_filter_popover.hide()
     
-def _columns_count_item_selected(w, data):
-    gui.editor_window.media_list_view.columns_changed(data)
+def columns_count_launch_pressed(launcher, widget, event):
+    guipopover.columns_count_popupover_show(launcher, widget, _columns_count_item_selected)
+    
+def _columns_count_item_selected(action, new_value_variant):
+    columns = int(new_value_variant.get_string())
+    gui.editor_window.media_list_view.columns_changed(columns)
  
+    action.set_state(new_value_variant)
+    guipopover._columns_popover.hide()
+     
 def import_project_media():
     dialogs.load_project_dialog(_media_import_project_select_dialog_callback, None, _("Select Project for Media Import"))
     
@@ -1370,7 +1455,7 @@ def _do_create_selection_compound_clip(dialog, response_id, name_entry):
     # Create unique file path in hidden render folder
     folder = userfolders.get_render_dir()
     uuid_str = hashlib.md5(str(os.urandom(32)).encode('utf-8')).hexdigest()
-    write_file = folder + "/"+ uuid_str + ".xml"
+    write_file = folder + uuid_str + ".xml"
 
     dialog.destroy()
     
@@ -1383,7 +1468,7 @@ def _do_create_selection_compound_clip(dialog, response_id, name_entry):
     # Create tractor
     tractor = mlt.Tractor()
     multitrack = tractor.multitrack()
-    track0 = mlt.Playlist()
+    track0 = mlt.Playlist(PROJECT().profile)
     multitrack.connect(track0, 0)
     for i in range(0, len(clips)):
         clip = clips[i]
@@ -1427,7 +1512,7 @@ def _do_create_sequence_compound_clip(dialog, response_id, name_entry):
 
     media_name = name_entry.get_text()
     folder = userfolders.get_render_dir()
-    write_file = folder + "/"+ media_name + ".xml"
+    write_file = folder + media_name + ".xml"
 
     dialog.destroy()
 
@@ -1448,7 +1533,7 @@ def _do_create_sequence_compound_clip_from_selected(dialog, response_id, name_en
 
     media_name = name_entry.get_text()
     folder = userfolders.get_render_dir()
-    write_file = folder + "/"+ media_name + ".xml"
+    write_file = folder + media_name + ".xml"
 
     dialog.destroy()
 
@@ -1475,7 +1560,7 @@ def _do_create_sequence_freeze_frame_compound_clip(dialog, response_id, name_ent
     
     media_name = name_entry.get_text()
     folder = userfolders.get_render_dir()
-    write_file = folder + "/"+ media_name + ".xml"
+    write_file = folder + media_name + ".xml"
 
     dialog.destroy()
     
@@ -1518,7 +1603,9 @@ def _append_media_files(media_files):
         new_clip.clip_in = 0
         new_clip.clip_out = new_clip.get_length() - 1
         if new_clip.media_type == appconsts.IMAGE:
-            in_fr, out_fr, default_grfx_length = editorpersistance.get_graphics_default_in_out_length()
+            default_grfx_length = PROJECT().get_current_bin_graphics_default_length()
+            in_fr = (new_clip.get_length() - 1) // 2 - (default_grfx_length // 2)
+            out_fr = in_fr + default_grfx_length - 1
             new_clip.clip_in = in_fr
             new_clip.clip_out = out_fr
 
@@ -1540,33 +1627,13 @@ def _append_media_files(media_files):
     action.do_edit()
 
 # ------------------------------------ bins
-def bins_panel_popup_requested(event):
-    bin_menu = bin_popup_menu
-    
-    guiutils.remove_children(bin_menu)
+def bins_panel_popup_requested(widget, event):
+    guipopover.bins_panel_mouse_popover_show(widget, event.x, event.y, _bin_menu_item_selected)
 
-    bin_menu.add(guiutils.get_menu_item(_("Add Bin"), _bin_menu_item_selected, ("add bin", None)))
-    bin_menu.add(guiutils.get_menu_item(_("Delete Selected Bin"), _bin_menu_item_selected, ("delete bin", None)))
+def bin_hambuger_pressed(launcher, widget, event, data):
+    guipopover.bins_panel_widget_popover_show(launcher, widget, _bin_menu_item_selected)
 
-    guiutils.add_separetor(bin_menu)
-    
-    move_menu_item = Gtk.MenuItem(_("Move Bin"))
-    move_menu = Gtk.Menu()
-    move_menu.add(guiutils.get_menu_item(_("Up"), _bin_menu_item_selected, ("up bin", None)))
-    move_menu.add(guiutils.get_menu_item(_("Down"), _bin_menu_item_selected, ("down bin", None)))
-    move_menu.add(guiutils.get_menu_item(_("First"), _bin_menu_item_selected, ("first bin", None)))
-    move_menu.add(guiutils.get_menu_item(_("Last"), _bin_menu_item_selected, ("last bin", None)))
-    move_menu_item.set_submenu(move_menu)
-    bin_menu.add(move_menu_item)
-    move_menu_item.show()
-    
-    bin_menu.popup(None, None, None, None, event.button, event.time)    
-
-def bin_hambuger_pressed(widget, event):
-    bins_panel_popup_requested(event)
-
-def _bin_menu_item_selected(widget, data):
-    msg, bin_obj = data
+def _bin_menu_item_selected(action, variant, msg):
     if msg == "add bin":
         add_new_bin()
     elif msg == "delete bin":
@@ -1697,7 +1764,18 @@ def move_files_to_bin(new_bin, moved_files):
     
     gui.editor_window.bin_info.display_bin_info()
     
-    
+def reverse_current_bin_media_order():
+    current_bin().file_ids.reverse()
+    gui.media_list_view.fill_data_model()
+
+
+def _set_bin_grfx_default_length_callback(dialog, response_id, value_spin):
+    if response_id == Gtk.ResponseType.ACCEPT:
+        new_default_len = value_spin.get_value()
+        PROJECT().set_current_bin_graphics_default_length(int(new_default_len))
+
+    dialog.destroy()
+
 # ------------------------------------ sequences
 def change_edit_sequence():
     selection = gui.sequence_list_view.treeview.get_selection()
@@ -1716,23 +1794,12 @@ def change_edit_sequence():
     app.change_current_sequence(row)
 
 def sequences_hamburger_pressed(widget, event):
-    sequence_panel_popup_requested(event)
+    sequence_panel_popup_requested(widget, event)
 
-def sequence_panel_popup_requested(event):
-    sequence_menu = sequence_popup_menu
-    
-    guiutils.remove_children(sequence_menu)
+def sequence_panel_popup_requested(widget, event):
+    guipopover.sequence_panel_popover_show(widget, event.x, event.y, _sequence_menu_item_selected)
 
-    sequence_menu.add(guiutils.get_menu_item(_("Add New Sequence"), _sequece_menu_item_selected, ("add sequence", None)))
-    sequence_menu.add(guiutils.get_menu_item(_("Edit Selected Sequence"), _sequece_menu_item_selected, ("edit sequence", None)))
-    sequence_menu.add(guiutils.get_menu_item(_("Delete Selected Sequence"), _sequece_menu_item_selected, ("delete sequence", None)))
-    guiutils.add_separetor(sequence_menu)
-    sequence_menu.add(guiutils.get_menu_item(_("Create Container Clip from Selected Sequence"), _sequece_menu_item_selected, ("compound clip", None)))
-    
-    sequence_menu.popup(None, None, None, None, event.button, event.time)    
-
-def _sequece_menu_item_selected(widget, data):
-    msg, bin_obj = data
+def _sequence_menu_item_selected(action, variable, msg):
     if msg == "add sequence":
         add_new_sequence()
     elif msg == "delete sequence":
@@ -1774,7 +1841,6 @@ def _add_new_sequence_dialog_callback(dialog, response_id, widgets):
     sequence.VIDEO_TRACKS_COUNT = v_tracks
 
     # Add new sequence.
-    print("new seq", name)
     PROJECT().add_named_sequence(name)
 
     gui.sequence_list_view.fill_data_model()
@@ -1871,13 +1937,73 @@ def _change_track_count_dialog_callback(dialog, response_id, tracks_select):
     v_tracks, a_tracks = tracks_select.get_tracks()
     dialog.destroy()
 
+    _do_tracks_count_change(v_tracks, a_tracks)
+
+def add_video_track():
+    nv, na = PROJECT().c_seq.get_track_counts()
+    
+    if nv + na > appconsts.MAX_TRACKS - 1:
+        _show_max_tracks_info()
+        return
+
+    dialogs.confirm_track_add_delete(True, True, nv + 1, na, _add_delete_track_dialog_callback)
+
+def add_audio_track():
+    nv, na = PROJECT().c_seq.get_track_counts()
+    
+    if nv + na > appconsts.MAX_TRACKS - 1:
+        _show_max_tracks_info()
+        return
+        
+    dialogs.confirm_track_add_delete(True, False, nv, na + 1, _add_delete_track_dialog_callback)
+
+def delete_video_track():
+    nv, na = PROJECT().c_seq.get_track_counts()
+    if nv == 1:
+        _show_min_tracks_info(True)
+        return
+        
+    dialogs.confirm_track_add_delete(False, True, nv - 1, na, _add_delete_track_dialog_callback)
+
+def delete_audio_track():
+    nv, na = PROJECT().c_seq.get_track_counts()
+    if na == 0:
+        _show_min_tracks_info(False)
+        return
+        
+    dialogs.confirm_track_add_delete(False, False, nv, na - 1, _add_delete_track_dialog_callback)
+
+def _show_max_tracks_info():
+    primary_txt = _("Can't add a new Track")
+    secondary_txt = _("Sequence already has 21 tracks, which is the maximum number allowed.")
+    dialogutils.info_message(primary_txt, secondary_txt, gui.editor_window.window)
+
+def _show_min_tracks_info(is_video):
+    if is_video:
+        primary_txt = _("Can't delete Video Track")
+        secondary_txt = _("Sequence has to have one Video Track.")
+    else:
+        primary_txt = _("Can't delete Audio Track")
+        secondary_txt = _("There are no Audio Tracks to delete.")
+
+    dialogutils.info_message(primary_txt, secondary_txt, gui.editor_window.window)
+    
+def _add_delete_track_dialog_callback(dialog, response_id, v_tracks, a_tracks):
+    if response_id != Gtk.ResponseType.ACCEPT:
+        dialog.destroy()
+        return
+
+    dialog.destroy()
+    _do_tracks_count_change(v_tracks, a_tracks)
+
+def _do_tracks_count_change(v_tracks, a_tracks):
     cur_seq_index = PROJECT().sequences.index(PROJECT().c_seq)
     
     if len(PROJECT().c_seq.tracks[-1].clips) == 1: # Remove hidden hack trick blank so that is does not get copied 
         edit._remove_clip(PROJECT().c_seq.tracks[-1], 0)
 
     if current_sequence().compositing_mode == appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
-        # These sequnces have one compositor for every non-V1 video track that will be recreated after track count changed.
+        # These sequences have one compositor for every non-V1 video track that will be recreated after track count changed.
         current_sequence().destroy_compositors()
 
     new_seq = sequence.create_sequence_clone_with_different_track_count(PROJECT().c_seq, v_tracks, a_tracks)
@@ -1890,8 +2016,6 @@ def _change_track_count_dialog_callback(dialog, response_id, tracks_select):
         # Put track compositors back
         current_sequence().add_full_track_compositors()
 
-    # tlinerender.get_renderer().timeline_changed()
-
 def combine_sequences():
     dialogs.combine_sequences_dialog(_combine_sequences_dialog_callback)
 
@@ -2055,34 +2179,32 @@ def _get_sequence_import_range(import_seq):
     return (start_track_range, end_track_range)
 
 def _update_gui_after_sequence_import(): # This copied  with small modifications into projectaction.py for sequence imports, update there too if needed...yeah.
-    updater.update_tline_scrollbar() # Slider needs to adjust to possily new program length.
+    updater.update_tline_scrollbar() # Slider needs to adjust to possibly new program length.
                                      # This REPAINTS TIMELINE as a side effect.
     updater.clear_editor_panel()
 
-    current_sequence().update_edit_tracks_length() # Needed for timeline renderering updates
-    current_sequence().update_hidden_track_for_timeline_rendering() # Needed for timeline renderering updates
+    current_sequence().update_edit_tracks_length() # Needed for timeline rendering updates
+    current_sequence().update_hidden_track_for_timeline_rendering() # Needed for timeline rendering updates
     editorstate.PLAYER().display_inside_sequence_length(current_sequence().seq_len)
 
     updater. update_seqence_info_text()
 
-def compositing_mode_menu_launched(widget, event):
-    guiutils.remove_children(compositing_mode_menu)
-
-    comp_top_free = guiutils.get_image_menu_item(_("Compositors Free Move"), "top_down", change_current_sequence_compositing_mode_from_corner_menu)
-    comp_full_track = guiutils.get_image_menu_item(_("Standard Full Track"), "full_track_auto", change_current_sequence_compositing_mode_from_corner_menu)
+def compositing_mode_menu_launched(launcher, widget, event):
+    guipopover.compositing_mode_menu_show(launcher, widget,  change_current_sequence_compositing_mode_from_corner_menu)
     
-    comp_top_free.connect("activate", lambda w: change_current_sequence_compositing_mode_from_corner_menu(appconsts.COMPOSITING_MODE_TOP_DOWN_FREE_MOVE))
-    comp_full_track.connect("activate", lambda w: change_current_sequence_compositing_mode_from_corner_menu(appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK))
+def change_current_sequence_compositing_mode_from_corner_menu(action, new_value_variant):
+    if new_value_variant.get_string() == "topdown":
+        new_compositing_mode = appconsts.COMPOSITING_MODE_TOP_DOWN_FREE_MOVE
+    else:
+        new_compositing_mode = appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK
 
-    compositing_mode_menu.add(comp_top_free)
-    compositing_mode_menu.add(comp_full_track)
-
-    compositing_mode_menu.popup(None, None, None, None, event.button, event.time)
-    
-def change_current_sequence_compositing_mode_from_corner_menu(new_compositing_mode):
+    # If new is same as old, do nothing
     if current_sequence().compositing_mode == new_compositing_mode:
         return 
-        
+    
+    action.set_state(new_value_variant)
+    guipopover._compositing_mode_popover.hide()
+    
     dialogs.confirm_compositing_mode_change(_compositing_mode_dialog_callback, new_compositing_mode)
 
 def change_current_sequence_compositing_mode(menu_widget, new_compositing_mode):
@@ -2113,8 +2235,11 @@ def do_compositing_mode_change(new_compositing_mode):
     gui.comp_mode_launcher.set_pixbuf(new_compositing_mode) # pixbuf indexes correspond with compositing mode enums.
 
 # --------------------------------------------------------- pop-up menus
-def media_file_menu_item_selected(widget, data):
-    item_id, media_file, event = data
+def media_file_popover_mouse_right_pressed(widget, media_file, event):
+    guipopover.media_file_popover_show(media_file, widget, event.x, event.y, media_file_menu_item_selected)
+    
+def media_file_menu_item_selected(action, variant, data):
+    item_id, media_file = data
     if item_id == "File Properties":
         _display_file_info(media_file)
     if item_id == "Open in Clip Monitor":
@@ -2130,17 +2255,13 @@ def media_file_menu_item_selected(widget, data):
         delete_media_files()
     if item_id == "Render Proxy File":
         proxyediting.create_proxy_menu_item_selected(media_file)
-    if item_id == "Edit Container Data":
-        containerprogramedit.show_container_data_program_editor_dialog(media_file.container_data)
-    if item_id == "Save Container Data":
-        media_file.save_program_edit_info()
-    if item_id == "Load Container Data":
-        media_file.load_program_edit_info()
     if item_id == "Recreate Icon":
         (icon_path, length, info) = projectdata.thumbnailer.write_image(media_file.path)
         media_file.info = info
         media_file.icon_path = icon_path
         media_file.create_icon()
+    if item_id == "Replace":
+        replace_media_file(media_file)
         
 def _select_treeview_on_pos_and_return_row_and_column_title(event, treeview):
     selection = treeview.get_selection()
