@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2013 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@ import editorpersistance # Aug-2019- SvdB - BB
 import editorstate
 import gui
 import guicomponents
+import guipopover
 import guiutils
 from editorstate import current_sequence
 import renderconsumer
@@ -39,8 +40,6 @@ destroy_window_event_id = -1
 
 FFMPEG_VIEW_SIZE = (20, 20) # Text edit area height for render opts. Width is ignored in current layout.
 
-_hamburger_menu = Gtk.Menu()
-
 # ----------------------------------------------------------- dialogs
 def render_progress_dialog(callback, parent_window, frame_rates_match=True):
     dialog = Gtk.Dialog(_("Render Progress"),
@@ -68,9 +67,9 @@ def render_progress_dialog(callback, parent_window, frame_rates_match=True):
     if frame_rates_match == False:
         # Aug-2019 - SvdB - BB
         if editorpersistance.prefs.double_track_hights:
-            warning_icon = Gtk.Image.new_from_stock(Gtk.STOCK_DIALOG_WARNING, Gtk.IconSize.DND)
+            warning_icon = Gtk.Image.new_from_icon_name("dialog-warning", Gtk.IconSize.DND)
         else:
-            warning_icon = Gtk.Image.new_from_stock(Gtk.STOCK_DIALOG_WARNING, Gtk.IconSize.MENU)
+            warning_icon = Gtk.Image.new_from_icon_name("dialog-warning", Gtk.IconSize.MENU)
         warning_text = Gtk.Label(label=_("Project and Render Profile FPS values are not same. Rendered file may have A/V sync issues."))
         warning_box = Gtk.HBox(False, 2)
         warning_box.pack_start(warning_icon,False, False, 0)
@@ -466,7 +465,7 @@ def _get_rendered_reverse_clip_length(media_file, range_combo, speed):
 # ----------------------------------------------------------- widgets
 class RenderQualitySelector():
     """
-    Component displays quality option relevant for encoding slection.
+    Component displays quality option relevant for encoding selection.
     """
     def __init__(self):
         self.widget = Gtk.ComboBoxText()
@@ -618,7 +617,8 @@ def get_render_panel_left(render_widgets, render_clicked_cb, to_queue_clicked_cb
             render_panel.pack_start(Gtk.Label(), True, True, 0)
     else:
         render_panel.pack_start(encoding_panel, False, False, 0)
-        render_panel.pack_start(Gtk.Label(), True, True, 0)
+        if editorstate.SCREEN_HEIGHT > 800:
+            render_panel.pack_start(Gtk.Label(), True, True, 0)
     
     # Stop panel **width** popping when/if GPU renders added.
     render_panel.set_size_request(300, 30)
@@ -714,27 +714,6 @@ class RenderRangePanel():
         self.vbox.pack_start(range_selector, False, False, 0)
 
 
-"""
-class RenderTypePanel():
-    
-    def __init__(self, render_type_changed_callback, preset_selection_changed_callback):
-        self.type_label = Gtk.Label(label=_("Type:"))
-        
-        self.type_combo = Gtk.ComboBoxText() # filled later when current sequence known
-        self.type_combo.append_text(_("User Defined"))
-        self.type_combo.append_text(_("Preset File type"))
-        self.type_combo.set_active(0)
-        self.type_combo.connect('changed', lambda w: render_type_changed_callback())
-    
-        self.presets_selector = PresetEncodingsSelector(preset_selection_changed_callback)
-
-        self.vbox = Gtk.VBox(False, 2)
-        self.vbox.pack_start(guiutils.get_two_column_box(self.type_label,
-                                                         self.type_combo, 80), 
-                                                         False, False, 0)
-        self.vbox.pack_start(self.presets_selector.widget, False, False, 0)
-"""
-
 class RenderProfilePanel():
 
     def __init__(self, out_profile_changed_callback):
@@ -778,7 +757,7 @@ class RenderEncodingPanel():
     
     def __init__(self, extension_label):
         self.quality_selector = RenderQualitySelector()
-        self.quality_selector.widget.set_size_request(110, 34)
+        self.quality_selector.widget.set_size_request(110, 20)
         self.quality_selector.update_quality_selection(0)
         self.audio_desc = Gtk.Label()
         self.encoding_selector = RenderEncodingSelector(self.quality_selector,
@@ -855,7 +834,7 @@ class RenderArgsPanel():
         scroll_frame.add(sw)
 
         self.hamburger_launch = guicomponents.HamburgerPressLaunch(self.hamburger_launch_pressed)
-        self.hamburger_launch.connect_launched_menu(_hamburger_menu)
+        self.hamburger_launch.do_popover_callback = True
         guiutils.set_margins(self.hamburger_launch.widget,5,0,0,0)
 
         opts_buttons_row = Gtk.HBox(False)
@@ -872,25 +851,10 @@ class RenderArgsPanel():
         self.vbox.pack_start(scroll_frame, True, True, 0)
         self.vbox.pack_start(opts_buttons_row, False, False, 0)
 
-    def hamburger_launch_pressed(self, widget, event):
-        menu = _hamburger_menu
-        guiutils.remove_children(menu)
+    def hamburger_launch_pressed(self, launcher, widget, event, data):
+        guipopover.render_args_popover_show(launcher, widget, self.hamburger_item_activated)
 
-        menu.add(guiutils.get_menu_item(_("Load Render Args from a text file"), self.hamburger_item_activated, "load_from_file"))
-        menu.add(guiutils.get_menu_item(_("Save Render Args into a text file"), self.hamburger_item_activated, "save_to_from_file"))
-        
-        guiutils.add_separetor(menu)
-        
-        menu.add(guiutils.get_menu_item(_("Load Render Args from Current Encoding"), self.hamburger_item_activated, "load_from_selection"))
-
-        guiutils.add_separetor(menu)
-        
-        menu.add(guiutils.get_menu_item(_("Reset all Render Options to Defaults"), self.hamburger_item_activated, "reset_all"))
-
-        menu.show_all()
-        menu.popup(None, None, None, None, event.button, event.time)
-    
-    def hamburger_item_activated(self, widget, msg):
+    def hamburger_item_activated(self, action, variant, msg):
         if msg ==  "load_from_file":
             self.load_args_callback()
         elif msg == "save_to_from_file":
@@ -958,7 +922,7 @@ class RenderArgsRow():
         self.use_args_check = Gtk.CheckButton()
         self.use_args_check.connect("toggled", self.use_args_toggled)
         self.use_args_check.set_tooltip_text(_("Render using key=value rendering options"))
-        self.use_args_check.set_margin_left(2)
+        self.use_args_check.set_margin_start(2)
         
         self.args_info = Gtk.Label()
         self.args_info.set_sensitive(False)
@@ -1042,6 +1006,7 @@ class RenderArgsRow():
         guiutils.set_margins(self.pop_over_pane, 4,4,4,4)
         self.pop_over_pane.show_all()
         
+        
         # ----------------------------------------- build GUI
         self.box = Gtk.VBox(False, 2)
         self.box.pack_start(use_opts_row , False, False, 0)
@@ -1147,13 +1112,13 @@ class RenderArgsPanelSmall():
         self.use_args_check.connect("toggled", self.use_args_toggled)
 
         self.opts_save_button = Gtk.Button()
-        icon = Gtk.Image.new_from_stock(Gtk.STOCK_SAVE, Gtk.IconSize.MENU)
+        icon = Gtk.Image.new_from_icon_name("document-save", Gtk.IconSize.MENU)
         self.opts_save_button.set_image(icon)
         self.opts_save_button.connect("clicked", lambda w: save_args_callback())
         self.opts_save_button.set_sensitive(False)
     
         self.opts_load_button = Gtk.Button()
-        icon = Gtk.Image.new_from_stock(Gtk.STOCK_OPEN, Gtk.IconSize.MENU)
+        icon = Gtk.Image.new_from_icon_name("document-open", Gtk.IconSize.MENU)
         self.opts_load_button.set_image(icon)
         self.opts_load_button.connect("clicked", lambda w: load_args_callback())
 
